'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// watch and trigger file remove event
// see: https://github.com/webpack/webpack/issues/1533

var WatchRemoveEventPlugin = function () {
	function WatchRemoveEventPlugin() {
		(0, _classCallCheck3.default)(this, WatchRemoveEventPlugin);

		this.removedFiles = [];
	}

	(0, _createClass3.default)(WatchRemoveEventPlugin, [{
		key: 'apply',
		value: function apply(compiler) {
			var _this = this;

			compiler.removedFiles = [];

			if (!compiler.watchFileSystem) return;

			var watchFileSystem = compiler.watchFileSystem;
			var watch = watchFileSystem.watch;


			watchFileSystem.watch = function (files, dirs, missing, startTime, options, callback, callbackUndelayed) {
				var result = watch.call(watchFileSystem, files, dirs, missing, startTime, options, function () {
					compiler.removedFiles = _this.removedFiles;
					_this.removedFiles = [];
					callback.apply(undefined, arguments);
				}, callbackUndelayed);

				var watchpack = watchFileSystem.watcher;
				watchpack.fileWatchers.forEach(function (w) {
					w.on('remove', _this.onRemove.bind(_this, watchpack, w.path));
				});
				return result;
			};
		}
	}, {
		key: 'onRemove',
		value: function onRemove(watchpack, file) {
			this.removedFiles.push(file);
			watchpack.emit('remove', file);
			watchpack._onChange(file);
		}
	}]);
	return WatchRemoveEventPlugin;
}();

// monkeypatching watchpack module to fix
// https://github.com/webpack/watchpack/pull/33

exports.default = WatchRemoveEventPlugin;
var DirectoryWatcher = void 0;
try {
	DirectoryWatcher = require('webpack/node_modules/watchpack/lib/DirectoryWatcher');
} catch (err) {
	DirectoryWatcher = require('watchpack/lib/DirectoryWatcher');
}

/* eslint-disable */
var FS_ACCURENCY = 10000;

function withoutCase(str) {
	return str.toLowerCase();
}

DirectoryWatcher.prototype.setFileTime = function setFileTime(filePath, mtime, initial, type) {
	var now = Date.now();
	var old = this.files[filePath];

	this.files[filePath] = [initial ? Math.min(now, mtime) : now, mtime];

	// we add the fs accurency to reach the maximum possible mtime
	if (mtime) mtime = mtime + FS_ACCURENCY;

	if (!old) {
		if (mtime) {
			if (this.watchers[withoutCase(filePath)]) {
				this.watchers[withoutCase(filePath)].forEach(function (w) {
					if (!initial || w.checkStartTime(mtime, initial)) {
						w.emit("change", mtime);
					}
				});
			}
		}
	} else if (!initial && mtime && type !== "add") {
		if (this.watchers[withoutCase(filePath)]) {
			this.watchers[withoutCase(filePath)].forEach(function (w) {
				w.emit("change", mtime);
			});
		}
	} else if (!initial && !mtime) {
		delete this.files[filePath];
		if (this.watchers[withoutCase(filePath)]) {
			this.watchers[withoutCase(filePath)].forEach(function (w) {
				w.emit("remove");
			});
		}
	}
	if (this.watchers[withoutCase(this.path)]) {
		this.watchers[withoutCase(this.path)].forEach(function (w) {
			if (!initial || w.checkStartTime(mtime, initial)) {
				w.emit("change", filePath, mtime);
			}
		});
	}
};
/* eslint-enable */